@@ -1,5 +1,3 @@
-context("vec-new")
-
 test_that("atomic vectors are spliced", {
   lgl <- lgl(TRUE, c(TRUE, FALSE), list(FALSE, FALSE))
   expect_identical(lgl, c(TRUE, TRUE, FALSE, FALSE, FALSE))
@@ -68,8 +66,8 @@ test_that("atomic inputs are implicitly coerced", {
   expect_identical(lgl(10L, FALSE, list(TRUE, 0L, 0)), c(TRUE, FALSE, TRUE, FALSE, FALSE))
   expect_identical(dbl(10L, 10, TRUE, list(10L, 0, TRUE)), c(10, 10, 1, 10, 0, 1))
 
-  expect_error(lgl("foo"), "Can't convert a character vector to a logical vector")
-  expect_error(chr(10), "Can't convert a double vector to a character vector")
+  expect_error(lgl("foo"), "Can't convert a string to a logical vector")
+  expect_error(chr(10), "Can't convert a number to a character vector")
 })
 
 test_that("type errors are handled", {
